#include <iostream>
#include <iomanip>
#include <fstream>
#include <cmath>
#include <cstdio>
#include <cstdlib>
#include <string>
#include "cdf.h"


#define __ZMODE__


ofstream output( "gAttr.dat" );


void ErrorHandler( const CDFstatus &stat ){
	CDFstatus errstat;
	char errmesg[CDF_STATUSTEXT_LEN+1];

	if( stat != CDF_OK ){
		errstat = CDFerror( stat, errmesg );
		cerr << errmesg << endl;

		exit(1);
	}

	return;
}


string datatypeInquire( const long &datatype ){
	string type;

	switch( datatype ){
	case CDF_BYTE:
		type = "CDF_BYTE";
		break;
	case CDF_CHAR:
		type = "CDF_CHAR";
		break;
	case CDF_INT1:
		type = "CDF_INT1";
		break;
	case CDF_UCHAR:
		type = "CDF_UCHAR";
		break;
	case CDF_UINT1:
		type = "CDF_UINT1";
		break;
	case CDF_INT2:
		type = "CDF_INT2";
		break;
	case CDF_UINT2:
		type = "CDF_UINT2";
		break;
	case CDF_INT4:
		type = "CDF_INT4";
		break;
	case CDF_UINT4:
		type = "CDF_UINT4";
		break;
	case CDF_REAL4:
		type = "CDF_REAL4";
		break;
	case CDF_FLOAT:
		type = "CDF_FLOAT";
		break;
	case CDF_REAL8:
		type = "CDF_REAL8";
		break;
	case CDF_DOUBLE:
		type = "CDF_DOUBLE";
		break;
	case CDF_EPOCH:
		type = "CDF_EPOCH";
		break;
	default:
		cerr << "Invalid datatype.\n";
		exit(1);
	}

	return type;
}


// show data of ordinary type
template <class T>
void outAttr( ){
	char attrName[CDF_ATTR_NAME_LEN+1];
	CDFstatus stat;
	T attr;

	stat = CDFlib( GET_, gENTRY_DATA_, &attr, 
                       ATTR_NAME_, attrName, 
								 NULL_ );
	ErrorHandler( stat );

	output.setf( ios::left );
	output.precision(2);

	if( strlen( attrName ) <= 12 ){
		output << setw(15) << attrName << attr << endl;
	}else{
		output << attrName << "   " << attr << endl;
	}

	return;
}


// show data of char type
void outCharAttr( ){
	CDFstatus stat;
	char attrName[CDF_ATTR_NAME_LEN+1];
	char *attr;
	long numElems;

	stat = CDFlib( GET_, ATTR_NAME_, attrName, 
                       gENTRY_NUMELEMS_, &numElems, 
								 NULL_ );
	ErrorHandler( stat );

	attr = new char[numElems+1];

	stat = CDFlib( GET_, gENTRY_DATA_, attr, 
								 NULL_ );
	ErrorHandler( stat );

	attr[numElems] = '\0';

	output.setf( ios::left );

	if( strlen( attrName ) <= 12 ){
		output << setw(15) << attrName << attr << endl;
	}else{
		output << attrName << "   " << attr << endl;
	}

	return;
}


// show data of unsigned char type
void outUCharAttr( ){
	CDFstatus stat;
	char attrName[CDF_ATTR_NAME_LEN+1];
	unsigned char *attr;
	long numElems;

	stat = CDFlib( GET_, ATTR_NAME_, attrName, 
                       gENTRY_NUMELEMS_, &numElems, 
								 NULL_ );
	ErrorHandler( stat );

	attr = new unsigned char[numElems+1];

	stat = CDFlib( GET_, gENTRY_DATA_, attr, 
								 NULL_ );
	ErrorHandler( stat );

	attr[numElems] = '\0';

	output.setf( ios::left );

	if( strlen( attrName ) <= 12 ){
		output << setw(15) << attrName << attr << endl;
	}else{
		output << attrName << "   " << attr << endl;
	}

	return;
}


// show data of Epoch type
void outEpochAttr( ){
	CDFstatus stat;
	char attrName[CDF_ATTR_NAME_LEN+1];
	double attr_epoch;
	long year, mon, day, hour, min, sec, msec;

	stat = CDFlib( GET_, gENTRY_DATA_, &attr_epoch, 
                       ATTR_NAME_, &attrName, 
								 NULL_ );
	ErrorHandler( stat );

	EPOCHbreakdown( attr_epoch, &year, &mon, &day, &hour, &min, &sec, &msec );

	output.setf( ios::left );

	if( strlen( attrName ) <= 12 ){
		output << setw(15) << attrName;
	}else{
		output << attrName << "   ";
	}

	char time[30];
	sprintf( time, "%04d/%02d/%02d  %02d:%02d:%02d.%04d\0", 
					 year, mon, day, hour, min, sec, msec );
	output << time << "\n";

	return;
}


void gAttrInquire( const int &num ){
	long datatype;
	CDFstatus stat, errstat;
	char errmesg[CDF_STATUSTEXT_LEN+1];

	stat = CDFlib( SELECT_, gENTRY_, num, 
								 GET_, gENTRY_DATATYPE_, &datatype, 
								 NULL_ );

	if( stat != CDF_OK ){
		if( stat == NO_SUCH_ENTRY ){
			return;
		}else{
			errstat = CDFerror( stat, errmesg );
			cerr << errmesg << endl;
			exit(1);
		}
	}

	switch( datatype ){
	case CDF_CHAR:
		outCharAttr( );
		break;
	case CDF_UCHAR:
		outUCharAttr( );
		break;
// 2 byte int  short ǤΤ狼ʤɤ褦
	case CDF_INT2:
		outAttr<short>( );
		break;
	case CDF_UINT2:
		outAttr<unsigned short>( );
		break;
	case CDF_INT4:
		outAttr<short>( );
		break;
	case CDF_UINT4:
		outAttr<unsigned short>( );
		break;
	case CDF_REAL4:
	case CDF_FLOAT:
		outAttr<float>( );
		break;
	case CDF_REAL8:
	case CDF_DOUBLE:
		outAttr<double>( );
		break;
	case CDF_EPOCH:
		outEpochAttr( );
		break;
// ʲηϡɤηȤɤФ狼ʤΤα
	case CDF_BYTE:
	case CDF_INT1:
	case CDF_UINT1:
		cerr << "Sorry.\n"
				 << "Datatype " << datatypeInquire( datatype )
				 << " is not supported\n";
		exit(1);
	default:
		cerr << "Invalid datatype.\n";
		exit(1);
	}

	return;
}


int main( int argc, char* argv[] ){
// Open the file
	if( argc != 2 ){
		cerr << "Usage : cdfgetattr CDF_FILE_NAME\n";
		exit(1);
	}

	CDFid id;
	char *filename = argv[1];
	CDFstatus stat, errstat;
	char errmesg[CDF_STATUSTEXT_LEN+1];

	stat = CDFlib( OPEN_, CDF_, filename, &id, 
#ifdef __ZMODE__
								 SELECT_, CDF_zMODE_, zMODEon2, 
#endif
								 NULL_ );  // ե򳫤

	ErrorHandler( stat ); // Error check

	long numAttrs;

	stat = CDFlib( GET_, CDF_NUMATTRS_, &numAttrs, 
								 NULL_ );
	ErrorHandler( stat ); // Error check


	int i, j;
	long numgEntries, scope;

	output << "CDF gAttiribute data\n";

	for( i = 0 ; i < numAttrs ; i++ ){
		stat = CDFlib( SELECT_, ATTR_, i, 
									 GET_, ATTR_SCOPE_, &scope, 
									 NULL_ );
		ErrorHandler( stat );

		if( scope == GLOBAL_SCOPE ){
			stat = CDFlib( GET_, ATTR_NUMgENTRIES_, &numgEntries, 
										 NULL_ );

			for( j = 0 ; j < numgEntries ; j++ ) gAttrInquire( j );
		}
	}


	stat = CDFclose(id);

	return 0;
}
